' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "GLFW.INC"
#INCLUDE ONCE "GDIPUTILS.INC"

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL hr, nWidth, nHeight, running, frames, x, y AS LONG
   LOCAL t, t0, fps AS DOUBLE
   LOCAL szTitlestr AS ASCIIZ * 200
   LOCAL xrot, yrot, zrot AS SINGLE
   LOCAL strTextureData AS STRING
   LOCAL TextureWidth AS LONG
   LOCAL TextureHeight AS LONG
   LOCAL glHandle AS DWORD

   LOCAL float_x, float_y AS SINGLE
   LOCAL wiggle_count AS LONG
   LOCAL float_xb, float_yb AS SINGLE

   ' Initialize GLFW
   glfwInit

   ' Open OpenGL window
   IF ISFALSE glfwOpenWindow(640, 480, 0, 0, 0, 0, 16, 0, %GLFW_WINDOW) THEN
      glfwTerminate
      EXIT FUNCTION
   END IF

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("tim.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   ' Assign an OpenGL handle to this texture
   glGenTextures 1, glHandle

   ' Activate our newly created texture
   glEnable %GL_TEXTURE_2D

   ' Create the texture
   glBindTexture %GL_TEXTURE_2D, glHandle
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Specify the clear value for the depth buffer
   glClearDepth 1.0!
   ' Specify the value used for depth-buffer comparisons
   glDepthFunc %GL_LESS
   ' Enable depth comparisons and update the depth buffer
   glEnable %GL_DEPTH_TEST
   ' Select smooth shading
   glShadeModel %GL_SMOOTH

   ' The array for the rgpoints on the grid of our "wave"
   REDIM rgpoints(44, 44, 2) AS SINGLE

   FOR float_x = 0.0! TO 9.0! STEP 0.2!
      FOR float_y = 0.0! TO 9.0 STEP 0.2!
         rgpoints(Int(float_x * 5), Int(float_y * 5), 0) = float_x - 4.4!
         rgpoints(Int(float_x * 5), Int(float_y * 5), 1) = float_y - 4.4!
         rgpoints(Int(float_x * 5), Int(float_y * 5), 2) = SIN(((float_x * 5 * 8) / 360) * 3.14159! * 2)
      NEXT
   NEXT

   ' Enable sticky keys
   glfwEnable %GLFW_STICKY_KEYS

   ' Disable vertical sync (on cards that support it)
   glfwSwapInterval 0

   ' Main loop
   running = %TRUE
   frames = 0
   t0 = glfwGetTime

   DO WHILE running

      ' Get time and mouse position
      t = glfwGetTime
      glfwGetMousePos x, y
      ' Calculate and display FPS (frames per second)
      IF t - t0 > 1.0! OR frames = 0 THEN
         fps = frames / (t-t0)
         wsprintf szTitlestr, "NEHE Lesson 11 (%i FPS)", BYVAL fps
         glfwSetWindowTitle szTitlestr
         t0 = t
         frames = 0
      END IF
      frames = frames + 1

      '  Get window size (may be different than the requested size)
      glfwGetWindowSize nWidth, nHeight
      IF nHeight <= 0 THEN nHeight = 1

      ' Set viewport
      glViewport 0, 0, nWidth, nHeight

      ' Clear screen and depth buffers
      glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

      ' Select and setup the projection matrix
      glMatrixMode %GL_PROJECTION
      glLoadIdentity

      ' Calculate The Aspect Ratio Of The Window
      gluPerspective 45.0!, nWidth/nHeight, 0.1!, 100.0!

      ' Select the modelview matrix
      glMatrixMode %GL_MODELVIEW
      glLoadIdentity

      glTranslatef 0.0!, 0.0!, -12.0!

      glRotatef xrot, 1.0!, 0.0!, 0.0!
      glRotatef yrot, 0.0!, 1.0!, 0.0!
      glRotatef zrot, 0.0!, 0.0!, 1.0!

      glBindTexture %GL_TEXTURE_2D, glHandle

      glPolygonMode %GL_BACK, %GL_FILL
      glPolygonMode %GL_FRONT, %GL_LINE

      glBegin %GL_QUADS

      FOR x = 0 TO 43
         FOR y = 0 TO 43
            float_x = x / 44
            float_y = y / 44
            float_xb = (x + 1) / 44
            float_yb = (y + 1) / 44

            glTexCoord2f float_x, float_y
            glVertex3f rgpoints(x, y, 0), rgpoints(x, y, 1), rgpoints(x, y, 2)

            glTexCoord2f float_x, float_yb
            glVertex3f rgpoints(x,y + 1, 0), rgpoints(x, y + 1, 1), rgpoints(x,y + 1, 2)

            glTexCoord2f float_xb, float_yb
            glVertex3f rgpoints(x + 1,y + 1, 0), rgpoints(x + 1,y + 1, 1), rgpoints(x + 1,y + 1, 2)

            glTexCoord2f float_xb, float_y
            glVertex3f rgpoints(x + 1, y, 0), rgpoints(x + 1, y, 1), rgpoints(x + 1, y, 2)
         NEXT
      NEXT

      glEnd

      IF wiggle_count = 2 THEN
         FOR y = 0 TO 44
            rgpoints(44, y, 2) = rgpoints(0, y, 2)
         NEXT
         FOR x = 0 TO 43
            FOR y = 0 TO 44
               rgpoints(x, y, 2) = rgpoints(x + 1, y, 2)
            NEXT
         NEXT
         wiggle_count = 0
      END IF

      INCR wiggle_count

      xrot = xrot + 0.3!                         ' x axis rotation
      yrot = yrot + 0.2!                         ' y axis rotation
      zrot = zrot + 0.4!                         ' z axis rotation

      ' Swap buffers
      glfwSwapBuffers

      ' Check if the ESC key was pressed or the window was closed
      running = NOT glfwGetKey(%GLFW_KEY_ESC) AND glfwGetWindowParam(%GLFW_OPENED)

   LOOP

   ' Delete the texture
   IF glHandle THEN glDeleteTextures(1, glHandle)

   ' Close OpenGL window and terminate GLFW
   glfwTerminate

END FUNCTION
' =======================================================================================
